'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setting = exports.build = exports.use = exports.configuration = exports.getConfiguration = exports.getWebpackConfiguration = exports.setupBuild = undefined;

var _compose = require('mblock-utils/compose');

var _compose2 = _interopRequireDefault(_compose);

var _webpackBundleAnalyzer = require('webpack-bundle-analyzer');

var _setting = require('./setting');

var _setting2 = _interopRequireDefault(_setting);

var _modules = require('./modules');

var _modules2 = _interopRequireDefault(_modules);

var _plugins = require('./plugins');

var _plugins2 = _interopRequireDefault(_plugins);

var _configuration = require('./configuration');

var _configuration2 = _interopRequireDefault(_configuration);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isProduction = process.env.NODE_ENV === 'production';

var baseWebpackOption = function baseWebpackOption() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return {
    devtool: isProduction ? 'source-map' : 'cheap-module-eval-source-map',
    output: {
      // Tell webpack to include comments in bundles with information about the contained modules.
      pathinfo: !isProduction
    },
    resolve: {},
    module: {
      rules: []
    },
    plugins: options && options.analyzer ? [new _webpackBundleAnalyzer.BundleAnalyzerPlugin()] : [],
    stats: 'normal'
  };
};

var setupBuild = exports.setupBuild = function setupBuild() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return {
    options: baseWebpackOption(options),
    configuration: (0, _configuration2.default)(options),
    modules: _modules2.default,
    plugins: _plugins2.default
  };
};

// return only the webpack configuration
var getWebpackConfiguration = exports.getWebpackConfiguration = function getWebpackConfiguration(setup) {
  return setup.options;
};
var getConfiguration = exports.getConfiguration = function getConfiguration(setup) {
  return setup.configuration;
};

var pluginsList = [];
var configurationOptions = void 0;

var configuration = exports.configuration = function configuration() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (configurationOptions) throw Error('Configuration has already set');
  configurationOptions = setting(options);
};

var use = exports.use = function use() {
  for (var _len = arguments.length, obj = Array(_len), _key = 0; _key < _len; _key++) {
    obj[_key] = arguments[_key];
  }

  Array.prototype.push.apply(pluginsList, obj);
};

var build = exports.build = function build() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  configurationOptions && pluginsList.push(configurationOptions);
  pluginsList.unshift(getWebpackConfiguration);
  return _compose2.default.apply(undefined, pluginsList.concat([setupBuild]))(options);
};

var setting = exports.setting = _setting2.default;